Package: qtl
Version: 1.74
Date: 2025-12-08
Title: Tools for Analyzing QTL Experiments
Author: Karl W Broman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4914-6671>),
  Hao Wu [aut],
  Gary Churchill [ctb] (ORCID: <https://orcid.org/0000-0001-9190-9284>),
  Saunak Sen [ctb] (ORCID: <https://orcid.org/0000-0003-4519-6361>),
  Danny Arends [ctb] (ORCID: <https://orcid.org/0000-0001-8738-0162>),
  Robert Corty [ctb],
  Timothee Flutre [ctb],
  Ritsert Jansen [ctb],
  Pjotr Prins [ctb] (ORCID: <https://orcid.org/0000-0002-8021-9162>),
  Lars Ronnegard [ctb],
  Rohan Shah [ctb],
  Laura Shannon [ctb],
  Quoc Tran [ctb],
  Aaron Wolen [ctb],
  Brian Yandell [ctb] (ORCID: <https://orcid.org/0000-0002-8774-9377>),
  R Core Team [ctb]
Authors@R: c(person(given = c("Karl", "W"),
                    family = "Broman",
                    role = c("aut", "cre"),
                    email = "broman@wisc.edu",
                    comment=c(ORCID = "0000-0002-4914-6671")),
             person(given = "Hao",
                    family = "Wu",
                    role = "aut"),
             person(given = c("Gary"),
                    family = "Churchill",
                    role = "ctb",
                    comment=c(ORCID = "0000-0001-9190-9284")),
             person(given = "Saunak",
                    family = "Sen",
                    role = "ctb",
                    comment=c(ORCID = "0000-0003-4519-6361")),
             person(given = "Danny",
                    family = "Arends",
                    role = "ctb",
                    comment=c(ORCID = "0000-0001-8738-0162")),
             person(given = "Robert",
                    family = "Corty",
                    role = "ctb"),
             person(given = "Timothee",
                    family = "Flutre",
                    role = "ctb"),
             person(given = "Ritsert",
                    family = "Jansen",
                    role = "ctb"),
             person(given = "Pjotr",
                    family = "Prins",
                    role = "ctb",
                    comment=c(ORCID = "0000-0002-8021-9162")),
             person(given = "Lars",
                    family = "Ronnegard",
                    role = "ctb"),
             person(given = "Rohan",
                    family = "Shah",
                    role = "ctb"),
             person(given = "Laura",
                    family = "Shannon",
                    role = "ctb"),
             person(given = "Quoc",
                    family = "Tran",
                    role = "ctb"),
             person(given = "Aaron",
                    family = "Wolen",
                    role = "ctb"),
             person(given = "Brian",
                    family = "Yandell",
                    role = "ctb",
                    comment=c(ORCID = "0000-0002-8774-9377")),
             person("R Core Team",
                    role = "ctb"))
Maintainer: Karl W Broman <broman@wisc.edu>
Description: Analysis of experimental crosses to identify genes
  (called quantitative trait loci, QTLs) contributing to variation in
  quantitative traits.
  Broman et al. (2003) <doi:10.1093/bioinformatics/btg112>.
Copyright: Code for Brent's method for finding the root of a univariate
        function was taken from R 2.15.1 (Copyright 1999, 2001 The R
        Core Team)
Depends: R (>= 2.14.0)
Imports: parallel, graphics, stats, utils, grDevices
Suggests: testthat
License: GPL-3
URL: https://rqtl.org, https://github.com/kbroman/qtl
BugReports: https://github.com/kbroman/qtl/issues
Encoding: UTF-8
ByteCompile: true
NeedsCompilation: yes
Packaged: 2025-12-08 13:03:51 UTC; kbroman
Repository: CRAN
Date/Publication: 2025-12-08 17:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:14:05 UTC; unix
Archs: qtl.so.dSYM
